/*

	Copyright  2003 Frank Condello
	
	Portions Copyright  2001 Dair Grant <mailto:dair@webthing.net>
	and Copyright  2003 Joe Strout <joe@strout.net> (from "LoadSaveJPEG")
	
	"GetData" based on code by Will Leshner <leshner@ljug.com>
	
	Linux port contributed by Jonathan Johnson <jonj@realsoftware.com>
	
*/

/* MachoGlue.cpp: Allows us to call Mach-O functions from a CFM application */

#include <CFBundle.h>
#include <Folders.h>
#include "MachoGlue.h"

Boolean machFuncsAvailable = false;

// CoreGraphics functions
CGBitmapContextGetDataPtr CGBitmapContextGetData = nil;
CGBitmapContextGetWidthPtr CGBitmapContextGetWidth = nil;
CGBitmapContextGetHeightPtr CGBitmapContextGetHeight = nil;
CGBitmapContextGetBytesPerRowPtr CGBitmapContextGetBytesPerRow = nil;
CGBitmapContextGetBitsPerPixelPtr CGBitmapContextGetBitsPerPixel = nil;


/* Ripped from Apple's CallMachOFramework example */
static OSStatus LoadFrameworkBundle(CFStringRef framework, CFBundleRef *bundlePtr)
{
	OSStatus 	err;
	FSRef 		frameworksFolderRef;
	CFURLRef	baseURL;
	CFURLRef	bundleURL;
	
	if (bundlePtr == nil)
		return -1;
	
	*bundlePtr = nil;
	
	baseURL = nil;
	bundleURL = nil;
	
	err = FSFindFolder(kOnAppropriateDisk, kFrameworksFolderType, true, &frameworksFolderRef);
	if (err == noErr) {
		baseURL = CFURLCreateFromFSRef(kCFAllocatorSystemDefault, &frameworksFolderRef);
		if (baseURL == nil) {
			err = coreFoundationUnknownErr;
		}
	}
	if (err == noErr) {
		bundleURL = CFURLCreateCopyAppendingPathComponent(kCFAllocatorSystemDefault, baseURL, framework, false);
		if (bundleURL == nil) {
			err = coreFoundationUnknownErr;
		}
	}
	if (err == noErr) {
		*bundlePtr = CFBundleCreate(kCFAllocatorSystemDefault, bundleURL);
		if (*bundlePtr == nil) {
			err = coreFoundationUnknownErr;
		}
	}
	if (err == noErr) {
	    if ( ! CFBundleLoadExecutable( *bundlePtr ) ) {
			err = coreFoundationUnknownErr;
	    }
	}
	
	if (err != noErr && *bundlePtr != nil) {
		CFRelease(*bundlePtr);
		*bundlePtr = nil;
	}
	if (bundleURL != nil) {
		CFRelease(bundleURL);
	}	
	if (baseURL != nil) {
		CFRelease(baseURL);
	}	
	
	return err;
}


/* Get the required function pointers */
Boolean InitMachoFunctionPointers (void)
{
	CFBundleRef		appServicesBundle;

	// Make sure the framework is available...
	if ( LoadFrameworkBundle(CFSTR("ApplicationServices.framework"), &appServicesBundle) != noErr )
		return false;
	
	// Fill in the pointers...
	
	if ( (CGBitmapContextGetData = (CGBitmapContextGetDataPtr) CFBundleGetFunctionPointerForName( appServicesBundle, CFSTR("CGBitmapContextGetData") )) == nil )
		return false;
	
	if ( (CGBitmapContextGetWidth = (CGBitmapContextGetWidthPtr) CFBundleGetFunctionPointerForName( appServicesBundle, CFSTR("CGBitmapContextGetWidth") )) == nil )
		return false;

	if ( (CGBitmapContextGetHeight = (CGBitmapContextGetHeightPtr) CFBundleGetFunctionPointerForName( appServicesBundle, CFSTR("CGBitmapContextGetHeight") )) == nil )
		return false;
	
	if ( (CGBitmapContextGetBytesPerRow = (CGBitmapContextGetBytesPerRowPtr) CFBundleGetFunctionPointerForName( appServicesBundle, CFSTR("CGBitmapContextGetBytesPerRow") )) == nil )
		return false;
	
	if ( (CGBitmapContextGetBitsPerPixel = (CGBitmapContextGetBitsPerPixelPtr) CFBundleGetFunctionPointerForName( appServicesBundle, CFSTR("CGBitmapContextGetBitsPerPixel") )) == nil )
		return false;
		
	return true;
}
