/*	NAME:
		MSL Allocator.c

	DESCRIPTION:
		Override for MSL allocator routines, uses temp mem.

	COPYRIGHT:
		Copyright  1998-2001 Dair Grant <mailto:dair@webthing.net>
	___________________________________________________________________________
*/
//============================================================================
//      Include files
//----------------------------------------------------------------------------
#include <Memory.h>
#include <Types.h>

#include "MSL Allocator.h"





//============================================================================
//      Internal constants
//----------------------------------------------------------------------------
#define kAppHeapThreshold                    (30 * 1024)    // Heap threshold





//=============================================================================
//      __sys_alloc : Allocate a block of memory.
//-----------------------------------------------------------------------------
void *
__sys_alloc(__std(size_t) theSize)
{   UInt32      totalMem, memAvail, requiredSize;
    Handle      theHnd, *hndPtr;
    OSErr       theErr;



    // Check our parameters
    if (theSize == 0)
        return(NULL);



    // Work out how much memory we need.
    requiredSize = theSize + sizeof(Handle);



    // Try and allocate some memory from temp mem first.
    // If that fails, we compact the app heap and try there.
    theHnd = TempNewHandle(requiredSize, &theErr);
    if (theHnd == NULL)
        {
        PurgeSpace((long *) &totalMem, (long *) &memAvail);
        if (requiredSize + kAppHeapThreshold <= memAvail)
            {
            PurgeMem(requiredSize);       
            theHnd = NewHandle(requiredSize);
            }
        }



    // If we didn't get the handle, we're done.
    if (theHnd == NULL)
        return(NULL);



    // Write the handle into the first four bytes, and return the
    // rest of the block as the requested memory.
    MoveHHi(theHnd);
    HLock(theHnd);
    hndPtr    = (Handle *) *theHnd;
    *hndPtr++ = theHnd;

    return((void *) hndPtr);
}





//=============================================================================
//      __sys_free : Free a block of memory.
//-----------------------------------------------------------------------------
void
__sys_free(void *thePtr)
{   Handle      theHnd, *hndPtr;



    // Chec our parameters
    if (thePtr == NULL)
        return;



    // Recover the handle from the pointer and free the block.
    hndPtr = (Handle *) thePtr;
    theHnd = *(hndPtr - 1);
    
    DisposeHandle(theHnd);
}



