/*

	Copyright  2003 Frank Condello
	
	Portions Copyright  2001 Dair Grant <mailto:dair@webthing.net>
	and Copyright  2003 Joe Strout <joe@strout.net> (from "LoadSaveJPEG")
	
	"GetData" based on code by Will Leshner <leshner@ljug.com>
	
	Linux port contributed by Jonathan Johnson <jonj@realsoftware.com>

*/

#include "PNGu.h"

#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

FILE *OpenFILEFromFolderItem( REALfolderItem theFile, const char *fileMode )
{
	REALstring	path = NULL;
	FILE		*ansiFile = NULL;
	
	if (theFile)
	{
		path = REALConvertString( REALpathFromFolderItem(theFile), REALGetFontEncoding("System") );
		if (path) ansiFile = fopen(path->CString(), fileMode);
	}
	
	return ansiFile;
}

Boolean BuildImageDataFromPicture(REALpicture thePicture, UInt32 theWidth, UInt32 theHeight, UInt8 *imageData)
{
	UInt32 x, y, dstRowBytes, srcRowBytes;
	UInt8 *dstPtr;
	REALpictureDescription desc;

	if (!REALLockPictureDescription( thePicture, &desc, pictureGdkPixmap ))
		return false;

	GdkPixbuf *pixBuf = gdk_pixbuf_new( GDK_COLORSPACE_RGB, true, 8, theWidth, theHeight );

	gdk_pixbuf_get_from_drawable( pixBuf, (GdkPixmap *)desc.pictureData, NULL, 0, 0, 0, 0, theWidth, theHeight );

	unsigned char *pixels = gdk_pixbuf_get_pixels( pixBuf );

	if (!pixels)
		return false;

	dstPtr = imageData;
	dstRowBytes = (theWidth * 3);
	srcRowBytes = (theWidth * 4);

	for (y = 0; y < theHeight; y++) {
		for (x = 0; x < theWidth; x++) {
			long sourceIndex = y * srcRowBytes + x * 4;
			dstPtr[(x * 3) + 0] = pixels[sourceIndex++];
			dstPtr[(x * 3) + 1] = pixels[sourceIndex++];
			dstPtr[(x * 3) + 2] = pixels[sourceIndex  ];
		}

		dstPtr += dstRowBytes;
	}

	g_object_unref( pixBuf );
	REALUnlockPictureDescription( thePicture );

	return true;
}

Boolean BuildImageDataFromPictureAndMask(REALpicture thePicture, REALpicture theMask, UInt32 theWidth, UInt32 theHeight, UInt8 *imageData)
{
	UInt32 x, y, dstRowBytes, srcRowBytes;
	UInt8 *dstPtr, *srcPtr;
	REALpictureDescription desc;


	if (!REALLockPictureDescription( thePicture, &desc, pictureGdkPixmap ))
		return false;

	GdkPixbuf *pixBuf = gdk_pixbuf_new( GDK_COLORSPACE_RGB, true, 8, theWidth, theHeight );

	gdk_pixbuf_get_from_drawable( pixBuf, (GdkPixmap *)desc.pictureData, NULL, 0, 0, 0, 0, theWidth, theHeight );

	unsigned char *pixels = gdk_pixbuf_get_pixels( pixBuf );

	if (!pixels)
		return false;

	dstPtr = imageData;
	dstRowBytes = (theWidth * 4);
	srcRowBytes = (theWidth * 4);

	for (y = 0; y < theHeight; y++) {
		for (x = 0; x < theWidth; x++) {
			long sourceIndex = y * srcRowBytes + x * 4;
			dstPtr[(x * 4) + 0] = pixels[sourceIndex++];
			dstPtr[(x * 4) + 1] = pixels[sourceIndex++];
			dstPtr[(x * 4) + 2] = pixels[sourceIndex  ];
			dstPtr[(x * 4) + 3] = 255;
		}

		dstPtr += dstRowBytes;
	}

	g_object_unref( pixBuf );
	REALUnlockPictureDescription( thePicture );

	// Now, fill in the 4th byte with the mask
	if (!REALLockPictureDescription( theMask, &desc, pictureGdkPixmap ))
		return false;

	pixBuf = gdk_pixbuf_new( GDK_COLORSPACE_RGB, true, 8, theWidth, theHeight );

	gdk_pixbuf_get_from_drawable( pixBuf, (GdkPixmap *)desc.pictureData, NULL, 0, 0, 0, 0, theWidth, theHeight );

	pixels = gdk_pixbuf_get_pixels( pixBuf );

	if (!pixels)
		return false;

	dstPtr = imageData;
    srcPtr = pixels;
	dstRowBytes = (theWidth * 4);
	srcRowBytes = (theWidth * 4);

	for (y = 0; y < theHeight; y++) {
		for (x = 0; x < theWidth; x ++) {
			dstPtr[(x * 4) + 3] = 255 - srcPtr[(x*4)+1];
		}

		dstPtr += dstRowBytes;
        srcPtr += srcRowBytes;
	}

	g_object_unref( pixBuf );
	REALUnlockPictureDescription( theMask );

	return true;
}